
# q-expansion of newform 1323.2.h.b, downloaded from the LMFDB on 16 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1323
weight = 2
poly_data = [1, 0, 0, -1, 0, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0], 1], [[0, 0, 0, 1, 0, 0], 1], [[0, 1, 0, 0, 0, 1], 1], [[0, 1, 1, 0, -1, 0], 1], [[0, 0, 0, 0, 1, -1], 1], [[0, 1, 0, 0, -1, -1], 1]]

hecke_ring_character_values = [[785, [-1, 1, 0, 0, 0, 0]], [1081, [-1, 1, 0, 0, 0, 0]]]
aps_data = [[-1, 0, 0, 1, 1, 0], [0, 0, 0, 0, 0, 0], [0, -1, 1, -1, -1, 1], [0, 0, 0, 0, 0, 0], [2, -2, -1, 1, 0, -1], [-1, 1, 4, -4, 0, 2], [0, -2, 1, 0, 0, 0], [-1, 1, -1, 1, 0, -2], [0, 4, 2, 1, 1, -1], [0, 3, -4, 5, 5, -5], [1, 0, 0, -6, -3, 0], [1, -1, 3, -3, 0, -3], [0, 0, 1, -1, 0, -1], [0, 1, 1, 1, 1, -1], [1, 0, 0, 5, 2, 0], [0, 2, 2, -3, -3, 3], [-1, 0, 0, -5, -5, 0], [-2, 0, 0, -3, -3, 0], [-4, 0, 0, 3, 0, 0], [-3, 0, 0, 3, -3, 0], [0, -7, 1, 4, 4, -4], [-7, 0, 0, 3, 0, 0], [0, 6, 1, 4, 4, -4], [-4, 4, 7, -7, 0, 3], [0, -1, 8, -7, -7, 7], [7, -7, 1, -1, 0, -2], [0, 2, 0, -3, -3, 3], [1, -1, -4, 4, 0, 0], [0, -5, 3, 0, 0, 0], [12, -12, -4, 4, 0, -2], [-4, 0, 0, -8, -13, 0], [0, 5, 10, -7, -7, 7], [5, -5, -7, 7, 0, -7], [5, -5, -5, 5, 0, -10], [0, -2, -7, 7, 7, -7], [1, -1, -12, 12, 0, -9], [1, 0, 0, -5, -10, 0], [-2, 2, -3, 3, 0, 0], [3, -3, 4, -4, 0, 14], [13, 0, 0, -10, -7, 0], [0, -7, -1, 0, 0, 0], [7, 0, 0, 6, 3, 0], [-1, 0, 0, -8, 1, 0], [5, 0, 0, -3, -3, 0], [-15, 0, 0, 3, -3, 0], [0, 2, 0, -3, -3, 3], [1, -1, 2, -2, 0, -2], [0, 5, -6, 6, 6, -6], [7, -7, -2, 2, 0, 1], [0, 5, 0, -12, -12, 12], [7, -7, 5, -5, 0, 9], [3, -3, 2, -2, 0, -8], [-10, 10, 0, 0, 0, -3], [-6, 0, 0, 15, 6, 0], [0, -16, -2, -4, -4, 4], [-1, 1, 17, -17, 0, 14], [0, 7, -2, 9, 9, -9], [8, -8, -3, 3, 0, -3], [-2, 2, 3, -3, 0, -6], [0, 18, -7, 8, 8, -8], [-5, 0, 0, 12, 15, 0], [-3, 3, -2, 2, 0, -7], [-2, 0, 0, -1, -8, 0], [-16, 0, 0, 4, 1, 0], [1, 0, 0, 7, 17, 0], [8, 0, 0, -8, -11, 0], [14, 0, 0, 5, 4, 0], [-5, 5, 12, -12, 0, -3], [-5, 0, 0, -7, 8, 0], [0, 8, 8, -10, -10, 10], [-8, 8, -2, 2, 0, -5], [-2, 2, 5, -5, 0, -6], [5, -5, 5, -5, 0, -5], [0, -5, 5, -1, -1, 1], [-1, 0, 0, -5, 5, 0], [0, 11, -11, -1, -1, 1], [-7, 7, -13, 13, 0, -4], [-19, 19, 3, -3, 0, -3], [0, 4, -7, -11, -11, 11], [4, 0, 0, 6, -12, 0], [12, -12, -8, 8, 0, -13], [0, 13, 15, -6, -6, 6], [0, 5, -4, -15, -15, 15], [-17, 0, 0, -4, 10, 0], [-11, 0, 0, 19, 11, 0], [2, 0, 0, 10, 4, 0], [0, 0, 0, -3, -6, 0], [-19, 0, 0, -3, 12, 0], [0, -6, 7, -2, -2, 2], [4, -4, 6, -6, 0, 3], [23, -23, -11, 11, 0, -3], [25, -25, 7, -7, 0, 4], [0, 7, -16, -1, -1, 1], [-9, 9, -7, 7, 0, 13], [0, -2, 18, -6, -6, 6], [-12, 0, 0, 18, 9, 0], [0, 8, -2, 11, 11, -11], [0, 13, -2, 6, 6, -6], [11, -11, 12, -12, 0, 15], [-11, 11, -2, 2, 0, -10], [0, -8, -20, 1, 1, -1], [0, -1, -4, 21, 21, -21], [-10, 0, 0, -8, 13, 0], [17, 0, 0, 4, 10, 0], [-13, 0, 0, -1, 13, 0], [0, -16, 0, -9, -9, 9], [0, -18, 13, 7, 7, -7], [-19, 19, 7, -7, 0, -6], [-14, 0, 0, 2, 14, 0], [0, -22, -1, 5, 5, -5], [0, 17, -8, 22, 22, -22], [0, 1, 24, -6, -6, 6], [24, -24, 11, -11, 0, -5], [-22, 22, 0, 0, 0, 15], [2, 0, 0, 16, -1, 0], [14, -14, -10, 10, 0, -13], [-19, 19, -2, 2, 0, -13], [0, 7, 19, -18, -18, 18], [0, -8, 23, -2, -2, 2], [0, -3, 5, -28, -28, 28], [-5, 0, 0, -17, 5, 0], [0, -14, 3, -9, -9, 9], [1, 0, 0, 23, -1, 0], [0, 32, 2, 0, 0, 0], [4, 0, 0, -6, 9, 0], [-6, 0, 0, 6, 12, 0], [2, 0, 0, 6, -6, 0], [11, -11, 16, -16, 0, 21], [0, -13, -19, 17, 17, -17], [0, 11, 12, 6, 6, -6], [0, 31, 0, -3, -3, 3], [-6, 6, 23, -23, 0, 10], [0, -11, 11, -16, -16, 16], [14, 0, 0, -5, 8, 0], [0, -10, 16, 2, 2, -2], [20, -20, 12, -12, 0, 9], [0, 1, 16, -12, -12, 12], [19, 0, 0, 6, 15, 0], [-24, 24, 19, -19, 0, 2], [0, 8, -7, 12, 12, -12], [4, 0, 0, 15, -9, 0], [5, 0, 0, 25, 22, 0], [32, 0, 0, -12, 0, 0], [-3, 0, 0, -21, -3, 0], [0, 5, 1, 10, 10, -10], [0, -9, 10, -8, -8, 8], [0, 11, -15, 0, 0, 0], [4, -4, -14, 14, 0, -17], [0, -31, 0, -12, -12, 12], [-2, 2, 11, -11, 0, 12], [0, -2, 9, 0, 0, 0], [33, 0, 0, 6, 21, 0], [5, 0, 0, -18, 3, 0], [0, -4, -26, 17, 17, -17], [-17, 17, -6, 6, 0, 18], [0, 9, -22, 11, 11, -11], [-8, 8, 9, -9, 0, -3], [25, 0, 0, -7, -19, 0], [-2, 0, 0, -1, 16, 0], [-31, 0, 0, 16, 1, 0], [8, 0, 0, -5, -17, 0], [-18, 0, 0, 12, 24, 0], [22, -22, 11, -11, 0, 28], [26, -26, 19, -19, 0, 18], [-17, 0, 0, 8, -16, 0], [-32, 32, 4, -4, 0, 13], [0, -35, -15, 0, 0, 0], [2, -2, -24, 24, 0, -12], [14, 0, 0, -11, 14, 0], [0, -13, 19, -4, -4, 4], [41, -41, 2, -2, 0, -1], [-28, 28, -3, 3, 0, 9], [0, 7, 8, -8, -8, 8], [19, -19, 4, -4, 0, 8], [-5, 0, 0, -12, -6, 0], [-18, 18, 31, -31, 0, 11], [0, 7, 7, 7, 7, -7], [0, -19, -1, 21, 21, -21], [16, 0, 0, 11, 31, 0], [37, 0, 0, 12, -3, 0], [2, 0, 0, -19, 7, 0], [0, 9, -29, 4, 4, -4], [-8, 8, -16, 16, 0, -14], [-19, 19, -11, 11, 0, 18], [10, 0, 0, -7, -1, 0], [19, -19, -23, 23, 0, -2], [0, 7, -9, 27, 27, -27], [14, -14, 5, -5, 0, 22], [0, 16, -21, -21, -21, 21], [0, 1, 19, 12, 12, -12], [-25, 25, -16, 16, 0, 1], [0, -15, 23, -25, -25, 25], [19, -19, 12, -12, 0, 27], [-20, 0, 0, 2, 5, 0], [0, 17, -1, -27, -27, 27], [32, 0, 0, 13, 1, 0], [-13, 0, 0, -31, -20, 0], [5, 0, 0, -15, -36, 0], [0, -36, 1, -8, -8, 8], [11, -11, -20, 20, 0, -6], [25, 0, 0, 14, -16, 0], [0, 5, 32, -10, -10, 10], [0, -43, 0, 3, 3, -3], [-16, 16, -15, 15, 0, -21], [6, 0, 0, -6, 27, 0], [0, 40, -5, -3, -3, 3], [26, -26, 24, -24, 0, 30], [0, 16, -4, -17, -17, 17], [0, 12, 8, -10, -10, 10], [-14, 0, 0, 18, -9, 0], [-14, 14, -15, 15, 0, 21], [21, -21, -5, 5, 0, 8], [0, -20, -2, -14, -14, 14], [10, 0, 0, -16, 11, 0], [2, 0, 0, 31, 34, 0], [-17, 0, 0, -21, -27, 0], [-28, 0, 0, -13, 19, 0], [25, -25, -17, 17, 0, 7], [40, -40, 5, -5, 0, -12], [21, -21, -4, 4, 0, 7], [0, -11, 0, 27, 27, -27], [-22, 22, -8, 8, 0, -4], [0, 16, -4, -20, -20, 20], [1, -1, -17, 17, 0, -7], [-6, 6, -8, 8, 0, -10], [0, -20, -29, -2, -2, 2], [28, 0, 0, -16, 11, 0], [0, 20, 2, -3, -3, 3], [34, 0, 0, 22, -1, 0], [5, 0, 0, -14, -23, 0], [11, 0, 0, -28, -17, 0], [0, -22, -8, 10, 10, -10], [-20, 20, -10, 10, 0, -2], [7, 0, 0, -7, -4, 0], [0, 2, 2, 20, 20, -20], [19, -19, 31, -31, 0, 43], [0, 50, 10, -2, -2, 2], [-2, 2, -4, 4, 0, -15], [33, -33, -13, 13, 0, -17], [-3, 0, 0, -3, 6, 0], [14, -14, -13, 13, 0, -7], [-25, 25, -10, 10, 0, -5], [16, 0, 0, 4, 23, 0], [7, -7, 36, -36, 0, 9], [6, -6, 22, -22, 0, 35], [10, 0, 0, 17, 2, 0], [7, 0, 0, 11, 1, 0], [2, 0, 0, -32, -29, 0], [17, 0, 0, 29, -14, 0], [3, 0, 0, -12, -18, 0], [16, -16, -1, 1, 0, -17], [-10, 10, 22, -22, 0, -12], [-38, 0, 0, 11, -1, 0], [0, 2, -7, -4, -4, 4], [19, -19, -5, 5, 0, -38], [4, -4, 11, -11, 0, 30], [0, -5, 15, -18, -18, 18], [41, -41, 21, -21, 0, 18], [-3, 0, 0, -12, -21, 0], [17, -17, -4, 4, 0, 28], [-26, 26, -14, 14, 0, -19], [0, 18, 5, -10, -10, 10], [16, 0, 0, 6, 21, 0], [7, 0, 0, -34, -14, 0], [-16, 0, 0, -8, 16, 0], [-2, 0, 0, 6, -24, 0], [3, 0, 0, 9, 12, 0], [0, 15, 1, 31, 31, -31], [16, -16, 21, -21, 0, -15], [46, 0, 0, -7, -1, 0], [0, -28, 13, 16, 16, -16], [0, -11, 9, 6, 6, -6], [5, -5, -6, 6, 0, -36], [0, 28, 33, -12, -12, 12], [-25, 25, -24, 24, 0, -27], [-31, 31, 6, -6, 0, 18], [0, 10, -10, 19, 19, -19], [1, -1, -3, 3, 0, -24], [-2, 2, -6, 6, 0, 12], [-35, 0, 0, 8, -7, 0], [17, 0, 0, -20, 4, 0], [26, 0, 0, 18, 27, 0], [0, 12, -20, 13, 13, -13], [0, -1, 32, 11, 11, -11], [0, -10, -15, 12, 12, -12], [7, -7, -19, 19, 0, 6], [0, 7, -21, 12, 12, -12], [-3, 3, 32, -32, 0, 31], [-7, 7, -18, 18, 0, 9], [-48, 0, 0, -9, 12, 0], [-52, 52, -10, 10, 0, 23], [0, -26, 4, 12, 12, -12], [0, 13, 5, 34, 34, -34], [-3, 3, 13, -13, 0, 8], [0, 22, -26, 37, 37, -37], [-37, 0, 0, -20, -17, 0], [-23, 0, 0, -15, -9, 0], [0, 30, 16, 10, 10, -10], [-31, 31, -17, 17, 0, 9], [0, -1, -28, 8, 8, -8], [0, -43, -12, 0, 0, 0], [-26, 26, -34, 34, 0, -51], [0, -5, -4, 5, 5, -5]]
